
.j01Tree {
    list-style-type: none;
    margin: 0px;
    padding: 0px;
} 

.j01Tree ul {
    margin-left: 12px;
    padding: 0px;
}

.j01Tree li {
    list-style: none;
    white-space: nowrap;
    margin: 0px;
    padding: 0px;
}

.j01Tree img {
    height: 16px;
}

.j01Tree a {
    text-decoration: none;
}

.j01Tree a:hover {
    font-weight: bold;
}

.j01Tree a.selected {
    font-weight: bold;
}

.j01Tree .j01TreeIcon {
    cursor: pointer;
}

.j01Tree li.j01TreeItem .j01TreeIcon {
    cursor: default;
}

.j01Tree li.j01TreeExpanded ul {
    display: block;
}

.j01Tree li.j01TreeCollapsed ul {
    display: none;
}

.j01TreeCollapsed > ul {
  visibility: hidden;
}

.j01TreeExpanded > ul {
  visibility: inherit;
}
